# include <QtGui>

# include "smallffmpegguiex.h"

MainWindow::MainWindow(QWidget *parent): QMainWindow(parent)
{
	setupUi(this);

	showMaximized();

	QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8"));
	QTimer::singleShot(0, this, SLOT(ffmpegtest()));

	vSuffixListe << ".cop" << ".avi" << ".avi" << ".mpg" << ".mpg" << ".mp4" << ".flv";
	aSuffixListe << ".cop" << ".wav" << ".mp3" << ".mp2" << ".ac3";

	batchTableWidget->horizontalHeader()->resizeSection(0,990);
	batchTableWidget->horizontalHeader()->resizeSection(1,60);
	batchTableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem("Jobs"));
	batchTableWidget->setHorizontalHeaderItem(1, new QTableWidgetItem("Status"));

	connect(actionOpenFile, SIGNAL(triggered()), this, SLOT(openDialog()));
	connect(actionSaveFile, SIGNAL(triggered()), this, SLOT(saveDialog()));

	connect(convertButton, SIGNAL(clicked()), this, SLOT(SingleConvert()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(abgebrochen()));

	connect(&dateiinfo, SIGNAL(readyReadStandardError()), this, SLOT(Information()));

	connect(&proc, SIGNAL(readyReadStandardError()), this, SLOT(updateprocbar()));
	connect(&proc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(procbeendet(int, QProcess::ExitStatus)));
	connect(&proc, SIGNAL(error(QProcess::ProcessError)), this, SLOT(processError(QProcess::ProcessError)));

	connect(&batch, SIGNAL(readyReadStandardError()), this, SLOT(updatebatchbar()));
	connect(&batch, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(batchbeendet(int, QProcess::ExitStatus)));
	connect(&batch, SIGNAL(error(QProcess::ProcessError)), this, SLOT(batchError(QProcess::ProcessError)));

	connect(vcodecComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setOutputFile()));
	connect(acodecComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setOutputFile()));

	connect(vcodecComboBox, SIGNAL(activated(int)), this, SLOT(vcopywahl()));
	connect(acodecComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(acopywahl()));
	connect(dvdButton, SIGNAL(clicked()), this, SLOT(dvdwahl()));

	connect(videoCheckBox, SIGNAL(stateChanged(int)), this, SLOT(VideoOnOff()));
	connect(audioCheckBox, SIGNAL(stateChanged(int)), this, SLOT(AudioOnOff()));

	connect(actionMoreFiles, SIGNAL(triggered()), this, SLOT(filesDialog()));
	connect(insertJobButton, SIGNAL(clicked()), this, SLOT(insertBatch()));
	connect(delJobButton, SIGNAL(clicked()), this, SLOT(delJob()));
	connect(batchConvertButton, SIGNAL(clicked()), this, SLOT(batchFirstStart()));
	connect(batchStopButton, SIGNAL(clicked()), this, SLOT(batchstop()));

	connect(actionBatchdateiLaden, SIGNAL(triggered()), this, SLOT(batchLoad()));
	connect(actionBatchdateiSpeichern, SIGNAL(triggered()), this, SLOT(batchSave()));

	connect(actionToolbarOnOff, SIGNAL(triggered()), this, SLOT(ToolbarOnOff()));
	connect(actionAbout, SIGNAL(triggered()), this, SLOT(showAutor()));

	connect(actionManual, SIGNAL(triggered()), this, SLOT(showHelp()));

}

void MainWindow::openDialog()
{
	QString initialDir = openLineEdit->text();
	if(initialDir.isEmpty())
		initialDir = QDir::homePath();

	QString InputFile = QFileDialog::getOpenFileName(
		this,
		"Open File Dialog",
		initialDir,
		"video (*.avi *.mpg *.mpeg *.flv *.wmv *.mp4 *.mov);;All Files(*.*)");

	QString OutputFile = saveLineEdit->text();

	if(!InputFile.isEmpty())
		{
		infoTextEdit->clear();
		openLineEdit->setText(InputFile);
		OutputFile = getOutputFile(InputFile,OutputFile);
		saveLineEdit->setText(OutputFile);

		QStringList arg;
		arg << "-i" << InputFile;

		stdausgabe.clear();
		dateiinfo.start("ffmpeg", arg);

		convertButton->setEnabled(true);
		}
}

void MainWindow::Information()
{
	QByteArray data= dateiinfo.readAllStandardError();
	stdausgabe= stdausgabe+QString::fromLocal8Bit(data);

	int pos1,pos2;

	if(stdausgabe.contains("least one"))
		{
		pos1 = stdausgabe.indexOf("Duration");
		pos2 = stdausgabe.indexOf("\n",pos1+1);
		QString duration = stdausgabe.mid(pos1,pos2-pos1);

		pos1 = stdausgabe.indexOf("Stream");
		pos2 = stdausgabe.indexOf("\n",pos1+1);
		QString video = stdausgabe.mid(pos1,pos2-pos1);

		pos1 = stdausgabe.indexOf("Stream",pos2+1);
		pos2 = stdausgabe.indexOf("\n",pos1+1);
		QString audio = stdausgabe.mid(pos1,pos2-pos1);

		infoTextEdit->append(duration);
		infoTextEdit->append(video);
		infoTextEdit->append(audio);
		stdausgabe.clear();
		}
}

void MainWindow::saveDialog()
{
	QString InputFile = openLineEdit->text();
	QString initialDir = saveLineEdit->text();
	if(initialDir.isEmpty())
		initialDir = QDir::homePath();

	QString OutputFile = QFileDialog::getSaveFileName(
		this,
		"Save File Dialog",
		initialDir,
		"video (*.avi *.mpg *.mpeg *.flv *.wmv *.mp4 *.mov);;All Files(*.*)");

	if(!OutputFile.isEmpty())
		{
		if(!InputFile.isEmpty())
			OutputFile = getOutputFile(InputFile,OutputFile);
		saveLineEdit->setText(OutputFile);
		}
}

void MainWindow::setOutputFile()
{
	QString InputFile = openLineEdit->text();
	QString OutputFile = saveLineEdit->text();
	if(!InputFile.isEmpty())
		{
		OutputFile=getOutputFile(InputFile,OutputFile);
		saveLineEdit->setText(OutputFile);
		}
}

QString MainWindow::getOutputFile(QString Eingabe, QString Ausgabe)
{
	QString Kopie = "-Kopie";

	if(Ausgabe.isEmpty())
		Ausgabe = QFileInfo(Eingabe).absolutePath()+QDir::separator() + QFileInfo(Eingabe).completeBaseName();
	else
		Ausgabe = QFileInfo(Ausgabe).absolutePath()+QDir::separator() + QFileInfo(Ausgabe).completeBaseName();

	if(Ausgabe.contains(Kopie))
		Ausgabe.remove(Ausgabe.indexOf(Kopie), Kopie.size());

	QString inputFileEnd = QFileInfo(Eingabe).suffix();

	if(videoCheckBox->isChecked())
		{
		if(vcodecComboBox->currentIndex()==0)
			Ausgabe += "."+inputFileEnd;
		else
			Ausgabe += vSuffixListe[vcodecComboBox->currentIndex()];
		}
	else	{
		if(acodecComboBox->currentIndex()==0)
			Ausgabe += "."+inputFileEnd;
		else
			Ausgabe += aSuffixListe[acodecComboBox->currentIndex()];
		}

	if(Eingabe==Ausgabe)
		Ausgabe.insert(Ausgabe.indexOf(QFileInfo(Ausgabe).suffix())-1, QString("-Kopie"));

	return Ausgabe;
}

QStringList MainWindow::createArguments(QString sourceFile, QString targetFile)
{
	QString vsize=(vauflspinBox->text())+"x"+(hauflspinBox->text());
	QStringList vcodec;
	vcodec << "copy" << "huffyuv" << "mjpeg" << "mpeg1video" << "mpeg2video" << "mpeg4" << "flv";
	QStringList vframes;
	vframes << "30" << "25" << "24";
	QStringList vratio;
	vratio << "4:3" << "16:9";
	QString vbitrate=vbitrateSpinBox->text()+"k";

	QStringList acodec;
	acodec << "copy" << "pcm_s16le" << "libmp3lame" << "mp2" << "ac3";
	QStringList abitrate;
	abitrate << "256k" << "224k" << "192k" << "128k" << "64k";
	QStringList afreq;
	afreq << "48000" << "44100" << "22050";


	QStringList args;
	args << "-y" << "-i" << sourceFile;

	if(!videoCheckBox->isChecked())
		args << "-vn";
	else if (vcodecComboBox->currentIndex()==0)
		args << "-vcodec" << "copy";
	else
		{
		if(dvdButton->isChecked())
			{
			args << "-target" << "pal-dvd";
			args << "-aspect" << vratio[ratioComboBox->currentIndex()];
			args << "-b" << vbitrate;
			}
		else
			{
			args << "-s" << vsize;
			args << "-vcodec" << vcodec[vcodecComboBox->currentIndex()];
			args << "-r" << vframes[fpsComboBox->currentIndex()];
			args << "-aspect" << vratio[ratioComboBox->currentIndex()];
			args << "-b" << vbitrate;
			}
		}

	if(!audioCheckBox->isChecked())
		args << "-an";
	else	{
		args << "-acodec" << acodec[acodecComboBox->currentIndex()];
		if(freqComboBox->isEnabled())
			args << "-ar" << afreq[freqComboBox->currentIndex()];
		if (abitrateComboBox->isEnabled())
			args << "-ab" << abitrate[abitrateComboBox->currentIndex()];
		if (channelButtonGroup->isEnabled())
			{
			args << "-ac";
			if (stereoButton->isChecked())
				{
				args << "2";
				}
			else
				{
				args << "1";
				}
			}
		}
	args << targetFile;

	return args;

}

void MainWindow::SingleConvert()
{
	progressBar->setValue(0);
	lcdNumber->display("0");
	convertButton->setEnabled(false);
	batchConvertGroupBox->setEnabled(false);

	QString framesout=fpsComboBox->currentText();
	faktor=framesout.toDouble();

	QString InputFile = openLineEdit->text();
	QString OutputFile = saveLineEdit->text();

	QStringList args;
	args=createArguments(InputFile,OutputFile);

	proc.start("ffmpeg", args);

	QString statusmessage=tr("Konvertiere ")+InputFile;
	statusBar()->showMessage(statusmessage);

}

void MainWindow::updateprocbar()
{
	int stdoutleeren=0;

	QByteArray data= proc.readAllStandardError();
	stdausgabe = stdausgabe+QString::fromLocal8Bit(data);

	if(stdausgabe.contains("Duration:"))
		firsttime=1;
	
	if(firsttime==1)
	{
	QRegExp reg("..:..:..\\...");
	if(stdausgabe.contains(reg))
		{
		QString time=reg.cap();

		QString stunde=time.mid(0,2);
		QString minute=time.mid(3,2);
		QString sekunde=time.mid(6,5);

		double std=stunde.toDouble();
		double min=minute.toDouble();
		double sek=sekunde.toDouble();

		gesamtzeit=(std*3600)+(min*60)+sek;
		firsttime=0;
		}
	}

	QRegExp regtime("time=..:..:..\\...");
	if(stdausgabe.contains(regtime))
		{
		int posakt=stdausgabe.indexOf("time");
		QString aktzeit=stdausgabe.mid(posakt+5,11);
		QString akt_std=aktzeit.mid(0,2);
		QString akt_min=aktzeit.mid(3,2);
		QString akt_sek=aktzeit.mid(6);

		double tmp_std,tmp_min,tmp_sek;
		tmp_std=akt_std.toDouble();
		tmp_min=akt_min.toDouble();
		tmp_sek=akt_sek.toDouble();

		aktPosSek=(tmp_std*60*60)+(tmp_min*60)+tmp_sek;

		proz=aktPosSek*100/gesamtzeit;

		stdausgabe.chop(1);
		progressBar->setValue(proz);
		stdoutleeren=1;
		}

	else if(stdausgabe.contains("time"))
		{
		int posakt=stdausgabe.indexOf("time");
		QString aktzeit=stdausgabe.mid(posakt+5,5);
		aktPosSek=aktzeit.toDouble();
		proz=aktPosSek*100/gesamtzeit;

		stdausgabe.chop(1);
		progressBar->setValue(proz);
		stdoutleeren=1;
		}

	if(stdausgabe.contains("fps"))
		{
		int posframe=stdausgabe.indexOf("fps");
		QString framezahl=stdausgabe.mid(posframe+4,3);
		double framerate=framezahl.toDouble();

		if(framerate!=0)
			{
			double gesamtframes=gesamtzeit*faktor;
			double restframes=gesamtframes-(aktPosSek*faktor);
	
			double restsekunden=restframes/framerate;
			double tmp=restsekunden/60.0;
			int lcdmin=(int)tmp;
			int lcdsek=(int)restsekunden-(lcdmin*60);
	
			QString strmin;
			strmin.setNum(lcdmin);
			QString strsek;
			strsek.setNum(lcdsek);
			if(strsek.length()==1)
				strsek="0"+strsek;
	
			QString timerwert=strmin+":"+strsek;

			lcdNumber->display(timerwert);
			}
		stdoutleeren=1;
		}

	if(stdoutleeren==1)
		{
		stdausgabe.clear();
		stdoutleeren=0;
		}

}

void MainWindow::procbeendet(int exitCode, QProcess::ExitStatus exitStatus)
{
	convertButton->setEnabled(true);
	batchConvertGroupBox->setEnabled(true);

	if (exitStatus==QProcess::CrashExit)
		{
		lcdNumber->display("Error");
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX Fehler");
		msgBox.setText(tr("<b>ffmpeg ist abgestürzt!</b><br><br>"
			"Bitte versuchen sie die Konvertierung<br>"
			"erneut zu starten."));
		msgBox.exec();
		statusBar()->showMessage(tr("Fehler!"), 5000);
		}
	else if(exitCode==1)
		{
		lcdNumber->display("Error");
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX Fehler");
		msgBox.setText(tr("<b>ffmpeg meldete einen Fehler!</b><br><br>"
			"Bitte überprüfen sie die Einstellungen für die Umwandlung <br>"
			"anhand der Fehlerbeschreibung in den Details.<br>"));
		msgBox.setDetailedText(stdausgabe);
		msgBox.exec();
		statusBar()->showMessage(tr("Fehler!"), 5000);
		}
	else if(exitCode==255)
		{
		lcdNumber->display(tr("Cancel"));
		statusBar()->showMessage(tr("Abgebrochen!"));
		}
	else 
		{
		progressBar->setValue(100);
		statusBar()->showMessage(tr("Konvertierung beendet."));
		}
	stdausgabe.clear();
}

void MainWindow::processError(QProcess::ProcessError error)
{
	if(error==QProcess::FailedToStart)
		{
		lcdNumber->display("ERROR");
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX Fehler");
		msgBox.setText(tr("<b>ffmpeg konnte nicht gestartet werden!</b><br><br>"
			"Bitte versuchen sie die Konvertierung<br>"
			"erneut zu starten."));
		msgBox.exec();
		convertButton->setEnabled(true);
		batchConvertGroupBox->setEnabled(true);
		statusBar()->showMessage(tr("Fehler!"), 5000);
		}
}

void MainWindow::abgebrochen()
{
	proc.terminate();
}

void MainWindow::vcopywahl()
{
	if (vcodecComboBox->currentIndex()==0)
		{
		vauflspinBox->setEnabled(false);
		hauflspinBox->setEnabled(false);
		ratioComboBox->setEnabled(false);
		vbitrateSpinBox->setEnabled(false);
		fpsComboBox->setEnabled(false);
		}
	else
		{
		vauflspinBox->setEnabled(true);
		hauflspinBox->setEnabled(true);
		ratioComboBox->setEnabled(true);
		vbitrateSpinBox->setEnabled(true);
		fpsComboBox->setEnabled(true);
		}
}

void MainWindow::dvdwahl()
{
	if (dvdButton->isChecked())
		{
		vauflspinBox->setEnabled(false);
		vauflspinBox->setValue(720);
		hauflspinBox->setEnabled(false);
		hauflspinBox->setValue(576);
		vcodecComboBox->setEnabled(false);
		vcodecComboBox->setCurrentIndex(4);
		ratioComboBox->setEnabled(true);
		vbitrateSpinBox->setEnabled(true);
		fpsComboBox->setEnabled(false);
		fpsComboBox->setCurrentIndex(1);
		acodecComboBox->setCurrentIndex(4);
		abitrateComboBox->setEnabled(true);
		channelButtonGroup->setEnabled(true);
		freqComboBox->setCurrentIndex(0);
		freqComboBox->setEnabled(true);
		}
	else
		{
		vauflspinBox->setEnabled(true);
		hauflspinBox->setEnabled(true);
		vcodecComboBox->setEnabled(true);
		fpsComboBox->setEnabled(true);
		}
}

void MainWindow::acopywahl()
{
	if (acodecComboBox->currentIndex()==0)
		{
		abitrateComboBox->setEnabled(false);
		channelButtonGroup->setEnabled(false);
		freqComboBox->setEnabled(false);
		}
	else if(acodecComboBox->currentIndex()==1)
		{
		abitrateComboBox->setEnabled(false);
		channelButtonGroup->setEnabled(true);
		freqComboBox->setEnabled(true);
		}
	else
		{
		abitrateComboBox->setEnabled(true);
		channelButtonGroup->setEnabled(true);
		freqComboBox->setEnabled(true);
		}
}

void MainWindow::VideoOnOff()
{
	if(videoCheckBox->isChecked())
		videoFrame->setEnabled(true);
	else
		videoFrame->setEnabled(false);
	setOutputFile();
}

void MainWindow::AudioOnOff()
{
	if(audioCheckBox->isChecked())
		audioFrame->setEnabled(true);
	else
		audioFrame->setEnabled(false);
}

/*********************************************************************
*                       BATCHVERARBEITUNG                            *
**********************************************************************/

 bool caseInsensitiveLessThan(const QString &s1, const QString &s2)
 {
     return s1.toLower() < s2.toLower();
 }

void MainWindow::filesDialog()
{
	QString initialDir = openLineEdit->text();
	if(initialDir.isEmpty())
		initialDir = QDir::homePath();

	QStringList moreFiles = QFileDialog::getOpenFileNames(
		this,
		"Open Files Dialog",
		initialDir,
		"video (*.avi *.mpg *.mpeg *.flv *.wmv *.mp4 *.mov);;All Files(*.*)");

	if(!moreFiles.isEmpty())
		{
		qSort(moreFiles.begin(),moreFiles.end(),caseInsensitiveLessThan);

		batchTableWidget->clearContents();
		batchTableWidget->setRowCount(0);

		foreach (QString str, moreFiles)
			{
			QString InputFile=str;
			QString OutputFile;
			OutputFile=getOutputFile(InputFile,OutputFile);

			QStringList args;
			args=createArguments(InputFile,OutputFile);
			QString insertArg = args.join(",");

			batchTableWidget->insertRow(batchTableWidget->rowCount());
			QTableWidgetItem *newItem = new QTableWidgetItem;
			newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			newItem->setText(insertArg);
			batchTableWidget->setItem(batchTableWidget->rowCount()-1,0, newItem);
			}
		position=0;
		jobs = batchTableWidget->rowCount();
		batchConvertButton->setEnabled(true);
		}
}

void MainWindow::insertBatch()
{
	QString InputFile = openLineEdit->text();
	QString OutputFile = saveLineEdit->text();

	if(!InputFile.isEmpty() && !OutputFile.isEmpty())
		{
		QStringList args;
		args=createArguments(InputFile,OutputFile);
		QString insertArg = args.join(",");

		batchTableWidget->insertRow(batchTableWidget->rowCount());
		QTableWidgetItem *newItem = new QTableWidgetItem;
		newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		newItem->setText(insertArg);
		batchTableWidget->setItem(batchTableWidget->rowCount()-1,0, newItem);

		jobs = batchTableWidget->rowCount();
		for(int z=0;z<jobs;z++)
			{
			if(batchTableWidget->item(z,1)==0)
				{
				position=z;
				break;
				}
			}
		batchConvertButton->setEnabled(true);
		}
}

void MainWindow::delJob()
{
	for(int i=batchTableWidget->rowCount();i>=0;i--)
		{
		if(batchTableWidget->item(i,0)!=0)
			{
			if(batchTableWidget->item(i,0)->isSelected())
				batchTableWidget->removeRow(i);
			}
		}

	if(batchTableWidget->rowCount()==0)
		batchConvertButton->setEnabled(false);

		for(int z=0;z<jobs;z++)
			{
			if(batchTableWidget->item(z,1)==0)
				{
				position=z;
				break;
				}
			}

	jobs = batchTableWidget->rowCount();
}

void MainWindow::batchFirstStart()
{
	insertJobButton->setEnabled(false);
	delJobButton->setEnabled(false);
	batchConvertButton->setEnabled(false);
	singleConvertGroupBox->setEnabled(false);

	batchConvert();
}

void MainWindow::batchConvert()
{
	progressBar->setValue(0);
	lcdNumber->display("0");

	QString input = batchTableWidget->item(position,0)->text();

	QRegExp reg("-r,[2-3][0-9]");
	if(input.contains(reg))
		{
		QString tmp=reg.cap();
		QString framesout=tmp.mid(3,2);
		faktor=framesout.toDouble();
		}

	QStringList arg;
	arg = input.split(",");

	QString InputFile = arg[2];

	batch.start("ffmpeg", arg);

	QString statusmessage = tr("Konvertiere ")+InputFile;
	statusBar()->showMessage(statusmessage);

}

void MainWindow::updatebatchbar()
{	
	QByteArray data= batch.readAllStandardError();
	stdausgabe= QString::fromLocal8Bit(data);

	if(stdausgabe.contains("Duration:"))
		firsttime=1;
	
	if(firsttime==1)
	{
	QRegExp reg("..:..:..\\...");
	if(stdausgabe.contains(reg))
		{
		QString time=reg.cap();

		QString stunde=time.mid(0,2);
		QString minute=time.mid(3,2);
		QString sekunde=time.mid(6,5);

		double std=stunde.toDouble();
		double min=minute.toDouble();
		double sek=sekunde.toDouble();

		gesamtzeit=(std*3600)+(min*60)+sek;
		firsttime=0;
		}
	}

	QRegExp regtime("time=..:..:..\\...");
	if(stdausgabe.contains(regtime))
		{
		int posakt=stdausgabe.indexOf("time");
		QString aktzeit=stdausgabe.mid(posakt+5,11);
		QString akt_std=aktzeit.mid(0,2);
		QString akt_min=aktzeit.mid(3,2);
		QString akt_sek=aktzeit.mid(6);

		double tmp_std,tmp_min,tmp_sek;
		tmp_std=akt_std.toDouble();
		tmp_min=akt_min.toDouble();
		tmp_sek=akt_sek.toDouble();

		aktPosSek=(tmp_std*60*60)+(tmp_min*60)+tmp_sek;

		proz=aktPosSek*100/gesamtzeit;

		stdausgabe.chop(1);
		progressBar->setValue(proz);
		}

	else if(stdausgabe.contains("time"))
		{
		int posakt=stdausgabe.indexOf("time");
		QString aktzeit=stdausgabe.mid(posakt+5,5);
		aktPosSek=aktzeit.toDouble();
		proz=aktPosSek*100/gesamtzeit;

		stdausgabe.chop(1);
		progressBar->setValue(proz);
		}

	if(stdausgabe.contains("fps"))
		{
		int posframe=stdausgabe.indexOf("fps");
		QString framezahl=stdausgabe.mid(posframe+4,3);
		double framerate=framezahl.toDouble();

		if(framerate!=0)
			{
			double gesamtframes=gesamtzeit*faktor;
			double restframes=gesamtframes-(aktPosSek*faktor);
	
			double restsekunden=restframes/framerate;
			double tmp=restsekunden/60.0;
			int lcdmin=(int)tmp;
			int lcdsek=(int)restsekunden-(lcdmin*60);
	
			QString strmin;
			strmin.setNum(lcdmin);
			QString strsek;
			strsek.setNum(lcdsek);
			if(strsek.length()==1)
				strsek="0"+strsek;
	
			QString timerwert=strmin+":"+strsek;

			lcdNumber->display(timerwert);
			}
		}

}

void MainWindow::batchbeendet(int exitCode, QProcess::ExitStatus exitStatus)
{
	if (exitStatus==QProcess::CrashExit)
		{
		lcdNumber->display("Error");
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX Fehler");
		msgBox.setText(tr("<b>ffmpeg ist abgestürzt!</b><br><br>"
			"Bitte versuchen sie die Konvertierung<br>"
			"erneut zu starten."));
		msgBox.exec();
		statusBar()->showMessage(tr("Fehler!"), 5000);
		insertJobButton->setEnabled(true);
		delJobButton->setEnabled(true);
		batchConvertButton->setEnabled(true);
		singleConvertGroupBox->setEnabled(true);
		}
	else if(exitCode==1)
		{
		lcdNumber->display("Error");
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX Fehler");
		msgBox.setText(tr("<b>ffmpeg meldete einen Fehler!</b><br><br>"
			"Bitte überprüfen sie die Einstellungen für die Umwandlung <br>"
			"anhand der Fehlerbeschreibung in den Details.<br>"));
		msgBox.setDetailedText(stdausgabe);
		msgBox.exec();
		statusBar()->showMessage(tr("Fehler!"), 5000);
		insertJobButton->setEnabled(true);
		delJobButton->setEnabled(true);
		batchConvertButton->setEnabled(true);
		singleConvertGroupBox->setEnabled(true);
		}
	else if(exitCode==255)
		{
		lcdNumber->display(tr("Cancel"));
		statusBar()->showMessage(tr("Abgebrochen!"));
		insertJobButton->setEnabled(true);
		delJobButton->setEnabled(true);
		batchConvertButton->setEnabled(true);
		singleConvertGroupBox->setEnabled(true);
		}
	else 
		{
		progressBar->setValue(100);
		batchTableWidget->setItem(position,1, new QTableWidgetItem("Done."));
		if(position<(jobs-1))
			{
			position++;
			batchConvert();
			}
		else if(position==(jobs-1))
			{
			progressBar->setValue(100);
			statusBar()->showMessage(tr("Konvertierung beendet."));
			insertJobButton->setEnabled(true);
			delJobButton->setEnabled(true);
			singleConvertGroupBox->setEnabled(true);
			}
		}
}

void MainWindow::batchError(QProcess::ProcessError error)
{
	if(error==QProcess::FailedToStart)
		{
		lcdNumber->display("ERROR");
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX Fehler");
		msgBox.setText(tr("<b>ffmpeg konnte nicht gestartet werden!</b><br><br>"
			"Bitte versuchen sie die Konvertierung<br>"
			"erneut zu starten."));
		msgBox.exec();
		statusBar()->showMessage(tr("Fehler!"), 5000);
		insertJobButton->setEnabled(true);
		delJobButton->setEnabled(true);
		batchConvertButton->setEnabled(true);
		singleConvertGroupBox->setEnabled(true);
		}
}

void MainWindow::batchstop()
{
	batch.terminate();
}


void MainWindow::batchLoad()
{
	if(batchTableWidget->rowCount()>0)
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
		msgBox.setDefaultButton(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Question);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX");
		msgBox.setText(tr("<p>Durch Öffnen einer neuen Batchliste<br>"
			"werden die momentanen Batcheinträge gelöscht.</p>"
			"Möchten Sie eine neue Batchliste öffnen?"));
		int ret = msgBox.exec();
		if(ret==4194304)
			return;
		}

	if(batchDir.isEmpty())
		batchDir = QDir::homePath();

	QString batchOpenFile = QFileDialog::getOpenFileName(
		this,
		"Open Batch File",
		batchDir,
		"Batch-File (*.btc);;All Files(*.*)");

	if(!batchOpenFile.isEmpty())
		{
		batchDir = batchOpenFile;

		QFile datei(batchOpenFile);
		if(datei.open(QIODevice::ReadOnly | QIODevice::Text))
			{
			QApplication::setOverrideCursor(Qt::WaitCursor);
			batchTableWidget->clearContents();
			batchTableWidget->setRowCount(0);
	
			QTextStream in(&datei);
			while(!in.atEnd())
				{
				QString line = in.readLine();
				batchTableWidget->insertRow(batchTableWidget->rowCount());
				QTableWidgetItem *newItem = new QTableWidgetItem;
				newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
				newItem->setText(line);
				batchTableWidget->setItem(batchTableWidget->rowCount()-1,0, newItem);
				}
			}
		datei.close();
		QApplication::restoreOverrideCursor();

		position=0;
		jobs = batchTableWidget->rowCount();
		batchConvertButton->setEnabled(true);
		}

}

void MainWindow::batchSave()
{
	if(batchTableWidget->rowCount()==0)
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Information);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX");
		msgBox.setText(tr("Leider sind noch keine Einträge in der Batchliste,<br>"
			"daher kann sie nicht gespeichert werden."));
		msgBox.exec();
		}
	else	{
		if(batchDir.isEmpty())
			batchDir = QDir::homePath();
	
		QString batchSaveFile = QFileDialog::getSaveFileName(
			this,
			"Save Batch File",
			batchDir,
			"Batch-File (*.btc);;All Files(*.*)");

		if(!batchSaveFile.isEmpty())
		   {
		   batchDir = batchSaveFile;

		   if(!batchSaveFile.endsWith(".btc"))
			batchSaveFile=batchSaveFile+".btc";

		   QFile datei(batchSaveFile);
		   if(datei.open(QFile::WriteOnly | QFile::Truncate))
			{
			QTextStream out(&datei);
			
			QApplication::setOverrideCursor(Qt::WaitCursor);

			for(int i=0;i<batchTableWidget->rowCount();i++)
			   {
			   if(batchTableWidget->item(i,0)!=0)
				{
				QString inhalt = batchTableWidget->item(i,0)->text();
				if(!inhalt.isEmpty())
					out << inhalt << endl;
				}
			   }
			}
		   QApplication::restoreOverrideCursor();
		   datei.close();
		   }
		}
}

// ----------------------- End of Batch -------------------------


void MainWindow::showAutor()
{
	QMessageBox::about(this, tr("Über small-ffmpeg-gui-EX 2.2"),
		tr("<h2>small-ffmpeg-gui-EX 2.2</h2>"
		"<p style=\" font-family:'Sans Serif'; font-size:12pt;\">Geschrieben von Udo Bender 2011/2012.</p>"
		"<p align=\"center\"><img src=\":/pics/udo.jpg\" /></p>"
		"<p align=\"center\" style=\" font-family:'Sans Serif'; font-size:14pt;\"><a href=\"http://www.udosworld.de\">www.udosworld.de</a></p>"
		"<p style=\" font-family:'Sans Serif'; font-size:12pt;\">small-ffmpeg-gui-EX ist ein grafisches Frontend für ffmpeg.<br>"
		"Es ermöglicht die Konvertierung einzelner Files oder auch<br>"
		"ganzer Listen von Files.</p>"));
}

void MainWindow::showHelp()
{
	QString verzeichnis=QCoreApplication::applicationDirPath();
	verzeichnis.append("/doc/index.html");
	QUrl url(verzeichnis);
	url.setScheme("file");
	QDesktopServices::openUrl(url);
	showNormal();
}


void MainWindow::ToolbarOnOff()
{
	if(actionToolbarOnOff->isChecked())
		toolBar->show();
	else
		toolBar->hide();
}

// Test nach dem Starten ob ffmpeg verfügbar, wenn nicht mit QMessageBox beenden

void MainWindow::ffmpegtest()
{
	QFileInfo test("/usr/bin/ffmpeg");
	if(!test.exists())
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("small-ffmpeg-gui-EX Fehler");
		msgBox.setText(tr("<b>Fehler: ffmpeg nicht gefunden!</b><br><br>"
			"small-ffmpeg-gui-EX benötigt für den Betrieb<br>"
			"das Programm ffmpeg.<br>"
			"Bitte installieren Sie die Anwendung mit<br>"
			"<b>apt-get install ffmpeg</b>.<br>"
			"small-ffmpeg-gui-EX wird geschlossen."));
		msgBox.exec();
		QCoreApplication::exit(2);
		}
}

